"""
`htrc.workset`

Contains function to extract all volume IDs from a JSON-LD workset 
representation.

Will eventually be expanded to allow for querying based on arbitrary
ID and for update and removal of volumes from Workset.
"""
from __future__ import absolute_import, print_function
from future import standard_library
standard_library.install_aliases()

import unicodecsv as csv
from io import BytesIO
import json
import os.path
from pprint import pprint
import re
from urllib.request import urlopen
from urllib.parse import urlparse

from pyld import jsonld

def get_volumes(data):
    """
    Takes a data structure in the canonical HathiTrust JSON-LD format
    and expands the dataset. Traverses the edm:gathers relation to find
    all HT volume IDs.
    
    Returns a list of volume IDs for use with the `htrc.metadata` and
    `htrc.volume` modules.
    """

    # Remove all namespaces to ensure proper referencing
    data = jsonld.expand(data)

    # Build up the list of volumes. Because the JSON-LD `@graph` may
    # contain multiple worksets, this code uses a set representation
    # to ensure that duplicates are removed
    volumes = set()
    for obj in data:
        # retrieve list of entities gathered
        gathers = obj.get('http://www.europeana.eu/schemas/edm/gathers', [])
        gathers = [vol['@id'].replace('http://hdl.handle.net/2027/','') 
                      for vol in gathers]

        # Check if `gathers` has any elements to ensure we don't add []
        # to the list of volumes.
        if gathers:
            volumes.update(gathers)

    # return the list representation, maintains a more consistent interface
    return list(volumes)

def create_jsonld(volumes, title=None, curator=None):
    """
    Takes a list of volumes and exports a JSON-LD formated workset
    """
    if curator is None:
        import getpass
        curator = getpass.getuser()
    
    context = "http://emblematica.library.illinois.edu/test/worksetcontext.jsonld"

    graph = {'@type':'http://wcsa.htrc.illinois.edu/Workset'}
    GATHERS = "http://www.europeana.eu/schemas/edm/gathers"
    graph[GATHERS] = [{'@id' : "http://hdl.handle.net/2027/" + vol} 
                          for vol in volumes]
    graph['numItems'] = len(volumes)
    if curator:
        graph['curator'] = curator
    if title:
        graph['title'] = title

    return jsonld.compact(graph, context)

def load(filename):
    """
    Takes a filename and retrieves a list of volumes from the workset
    description. If a URL is passed, automatically uses `load_url` to resolve.
    """
    if filename.startswith('http://') or filename.startswith('https://'):
        return load_url(filename)

    with open(filename) as infile:
        data = json.load(infile)

    # Retrieve and print the volumes
    return get_volumes(data)


def load_url(url):
    """
    Takes a workset URL, parses it, and uses the workset retrieval API to fetch
    the data and return the volumes..
    """
    url_components = urlparse(url)
    if url_components.netloc.startswith('babel.hathitrust.org'):
        return load_hathitrust_collection(url)
    elif (url_components.netloc.startswith('htrc.hathitrust.org')
        and url_components.path.startswith('/wsid/')):
        base_url = 'http://acbres224.ischool.illinois.edu:8080'
        base_url += '/dcWSfetch/getDescription?id='
        base_url += url
        url = base_url
    elif (url_components.netloc.startswith('acbres224.ischool.illinois.edu')
        and url_components.path.startswith('/dcWSfetch/')):
        # copied from direct call to WS fetch, a-ok.
        pass
    else:
        raise ValueError("Invalid workset URL: {}".format(url))

    response = urlopen(url)
    data = json.loads(response.read().decode('utf-8'))

    return get_volumes(data)


def get_volumes_from_csv(data):
    """
    Retrieves the volume list for a given HathiTrust collection.
    """

    csvfile = BytesIO(data)
    reader = csv.DictReader(csvfile, delimiter='\t')
    volumes = [row['htitem_id'] for row in reader] 
    csvfile.close()

    return volumes 


def load_hathitrust_collection(url):
    """
    Retrieves the volume list for a given HathiTrust Collection URL.
    In contrast to `get_volumes_csv`, which makes the request and handles data,
    this function parses out the collection ID from a variety of canonical URL
    schemes for collections:
    - https://babel.hathitrust.org/shcgi/mb?a=listis;c=548413090
    - https://babel.hathitrust.org/cgi/mb?a=listis&c=548413090
    """
    if not url.startswith('https://babel.hathitrust.org/'):
        raise ValueError('Invalid HathiTrust Collection URL: {}'.format(url))
    try:
        collection_id = re.search('c=(\d+)', url).group(1)
    except AttributeError:
        raise ValueError('Invalid HathiTrust Collection URL: {}'.format(url))
    
    url = "https://babel.hathitrust.org/shcgi/mb"
    data = "a=download&c={}&format=text".format(collection_id)

    response = urlopen(url, bytes(data.encode('utf-8')))
    data = response.read()

    return get_volumes_from_csv(data)


def path_to_volumes(path):
    """
    Takes a path and resolves to a list of volumes.

    Accepts:
    - Plaintext file, each line is an ID
    - Directory with subfolders that are volume pages
    - JSON or JSONLD workset representation
    - HT CB or HTRC WCSA URL.
    """
    if os.path.isdir(path):
        volumes = [id for id in os.listdir(path) if not id.endswith('.log')]
    elif (path.endswith('json')
        or path.endswith('jsonld')
        or path.startswith('http://')
        or path.startswith('https://')):
        volumes = load(path)
    elif os.path.isfile(path):
        with open(path) as infile:
            volumes = [line.strip() for line in infile]
    else:
        raise ValueError("invalid workset path")

    return volumes



#!/usr/bin/env python
"""
`htrc.volumes`

Contains the configuration parser object.
"""
from future import standard_library
standard_library.install_aliases()
from builtins import input

from configparser import RawConfigParser as ConfigParser, NoSectionError
from codecs import open
import logging
import os.path
import shutil

from htrc.lib.cli import bool_prompt

DEFAULT_PATH = os.path.expanduser('~')
DEFAULT_PATH = os.path.join(DEFAULT_PATH, '.htrc')
if not os.path.exists(DEFAULT_PATH):
    DEFAULT_FILE = os.path.dirname(__file__)
    DEFAULT_FILE = os.path.join(DEFAULT_FILE, '.htrc.default')
    logging.info("Copying default config file to home directory.")
    shutil.copyfile(DEFAULT_FILE, DEFAULT_PATH)

def _get_value(section, key, path=None):
    if path is None:
        path = DEFAULT_PATH

    config = ConfigParser(allow_no_value=True)
    with open(path, encoding='utf8') as configfile:
        config.readfp(configfile)
    try:
        return config.get(section, key)
    except NoSectionError:
        raise EnvironmentError("Config not set for {} {} in {}".format(
            section, key, path))
    
def get_host_port(path=None):
    host = _get_value('data', 'host', path)
    port = _get_value('data', 'port', path)
    return (host, port)


def get_dataapi_epr(path=None):
    return _get_value('data', 'url', path)


def get_oauth2_url(path=None):
    return _get_value('oauth', 'url', path)


def get_oauth2_port(path=None):
    return _get_value('oauth', 'port', path)


def get_oauth2_host_port(path=None):
    host = _get_value('oauth', 'host', path)
    port = _get_value('oauth', 'port', path)
    return (host, port)


# Add jwt credential access methods
def get_jwt_token(path=None):
    return _get_value('jwt','token', path)


def get_credentials(path=None):
    """
    Either retrieves credentials from existing config file or prompts user.
    Very convenient function for CLI applications.
    """
    if path is None:
        path = DEFAULT_PATH

    try:
        username, password = credentials_from_config(path)
    except EnvironmentError:
        username, password = credential_prompt(path)

    return username, password
    

def credential_prompt(path=None):
    """
    A prompt for entering HathiTrust credentials.
    """
    print("Please enter your HathiTrust credentials.")
    username = input("Token: ")
    password = input("Password: ")
    save = bool_prompt("Save credentials?", default=True)

    if save:
        save_credentials(username, password, path)

    return (username, password)

def save_credentials(username, password, path=None):
    """
    Saves credentials in the config file.
    """
    # Default to ~/.htrc
    if path is None:
        path = DEFAULT_PATH

    # Open and modify existing config file, if it exists.
    config = ConfigParser(allow_no_value=True)
    if os.path.exists(path):
        config.read(path)
    if not config.has_section('main'):
        config.add_section('main')
    config.set('main', 'username', username)
    config.set('main', 'password', password)
    with open(path, 'w') as credential_file:
        config.write(credential_file)

    return (username, password)

def credentials_from_config(path):
    """
    Retrieves the username and password from a config file for the Data API.
    Raises an EnvironmentError if not specified.
    See also: credential_prompt
    """
    username = None
    password = None

    config = ConfigParser(allow_no_value=True)
    if os.path.exists(path):
        config.read(path)
        if config.has_section('main'):
            username = config.get("main", "username")
            password = config.get("main", "password")

    if not username and not password:
        logging.error("Config path: {}".format(path))
        raise EnvironmentError("No username and password stored in config file.")

    return (username, password)

def populate_parser(parser):
    return parser

if __name__ == '__main__':
    from argparse import ArgumentParser

    parser = ArgumentParser()
    parser = populate_parser(parser)
    parser.parse_args()




from builtins import str
import os, os.path
import subprocess
import tarfile
import wget

# Mallet is downloaded and intalled in users current directory
def install_mallet():
    if not os.path.exists("/home/dcuser/mallet"):
        os.makedirs('/home/dcuser/mallet')
        mallet_zip = wget.download('http://mallet.cs.umass.edu/dist/mallet-2.0.8RC3.tar.gz')
        mallet_dir = tarfile.open(mallet_zip, "r:gz")
        mallet_dir.extractall(path="/home/dcuser/mallet")
        mallet_dir.close()

def main(path, topics, iterations):
    install_mallet()

    # import the workset to MALLET format.
    subprocess.check_call([
        '/home/dcuser/mallet/mallet-2.0.8RC3/bin/mallet',
        'import-dir',
        '--input', path,
        '--output', os.path.join(path, '../corpus.mallet'),
        '--keep-sequence',
        '--remove-stopwords'
        ])

    subprocess.check_call([
        '/home/dcuser/mallet/mallet-2.0.8RC3/bin/mallet',
        'train-topics',
        '--input', os.path.join(path, '../corpus.mallet'),
        '--num-topics', str(topics),
        '--output-state', os.path.join(path, '../mallet_state.gz'),
        '--output-topic-keys', os.path.join(path, '../mallet_topic-keys.txt'),
        '--output-doc-topics', os.path.join(path, '../mallet_doc-topics.txt'),
        '--num-iterations', str(iterations)
        ])

def populate_parser(parser=None):
    if parser is None:
        from argparse import ArgumentParser
        parser = ArgumentParser()
    parser.add_argument('-k', help="number of topics", required=True)
    parser.add_argument('--iter', help="number of iterations", default=200)
    parser.add_argument('path', default='/media/secure_volume/workset/',
        nargs='?')
    return parser


if __name__ == '__main__':
    from argparse import ArgumentParser
    parser = ArgumentParser(description="MALLET tools for the HTRC")
    populate_parser(parser)
    args = parser.parse_args()

    main(args.path, args.k, args.iter)



from builtins import map
import subprocess

def main(path, topics, iterations):
    if path.endswith('/'):
        path = path[:-1]

    subprocess.check_call([
        'topicexplorer', 'init', path,
        '--name', '"HathiTrust Workset"',
        '--rebuild', '--htrc', '-q'
    ])
    subprocess.check_call([
        'topicexplorer', 'prep', path, 
        '-q', '--min-word-len', '3', '--lang', 'en'
    ])
    subprocess.check_call([
        'topicexplorer', 'train', path,
        '-k'] + list(map(str,topics)) + [
        '--iter', iterations,
        '--context-type', 'book',
        '-q'
    ])
    subprocess.check_call([
        'topicexplorer', 'launch', path 
    ])

def populate_parser(parser=None):
    if parser is None:
        from argparse import ArgumentParser
        parser = ArgumentParser()

    parser.add_argument('-k', type=int, nargs='+', required=True,
        help="number of topics")
    parser.add_argument('--iter', help="number of iterations", default=200)
    parser.add_argument('path', default='/media/secure_volume/workset/',
        nargs='?')
    return parser

if __name__ == '__main__':
    from argparse import ArgumentParser
    parser = ArgumentParser(description="Topic Explorer tools for the HTRC")
    populate_parser(parser)
    args = parser.parse_args()

    main(args.path, args.k, args.iter)



#!/usr/bin/env python
from setuptools import setup, find_packages
import os
import platform
import sys

__version__ = '0.1.28'

install_requires = [ 'PyLD', 'future', 'prov', 'unicodecsv', 'progressbar2',
    'requests', 'wget', 'sphinx-argparse', 'sphinxcontrib-fulltoc']
if sys.version_info.major == 2:
    install_requires.append('configparser')
    install_requires.append('mock')

setup(
    name='htrc',
    version=__version__,
    description='HathiTrust Research Center API Access',
    author = "HathiTrust Research Center",
    author_email = "htrc@indiana.edu",
    url='http://analytics.hathitrust.org',
    download_url='http://github.com/htrc/HTRC-PythonSDK',
    keywords = [],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing :: Linguistic",
        ],
    packages=find_packages(),
    install_requires=install_requires,
    include_package_data=True,
    data_files=[('htrc/mock/volumes/', ['htrc/mock/volumes/example.zip']),
                ('htrc/', ['htrc/.htrc.default'])],
    zip_safe=False,
    entry_points={
        'console_scripts' : ['htrc = htrc.__main__:main']
    },
    test_suite="unittest2.collector",
    tests_require=['unittest2']
)




from __future__ import print_function
from future import standard_library
standard_library.install_aliases()

import sys
if sys.version_info.major == 2:
    from mock import Mock, patch, PropertyMock
elif sys.version_info.major == 3:
    from unittest.mock import Mock, patch, PropertyMock

import unittest2 as unittest

from htrc.lib.cli import *

class TestVolumes(unittest.TestCase):
    @patch('htrc.lib.cli.input')
    def test_bool_prompt(self, input_mock):
        # test True
        input_mock.return_value = 'y'
        return_value = bool_prompt("Enter yes")
        self.assertEqual(return_value, True)
        
        input_mock.return_value = 'n'
        return_value = bool_prompt("Enter no")
        self.assertEqual(return_value, False)

        input_mock.return_value = ''
        return_value = bool_prompt("Enter nothing for false", default=False)
        self.assertEqual(return_value, False)
        
        return_value = bool_prompt("Enter nothing for true", default=True)
        self.assertEqual(return_value, True)

    @patch('htrc.lib.cli.input')
    def test_prompt_default(self, input_mock):
        input_mock.return_value = ''
        return_value = prompt("Enter nothing for 3", default='3')
        self.assertEqual(return_value, '3')

